/*****************************************************************
 *
 *  CommentBuf.h                    XMM Science Analysis Subsystem
 *
 *  streambuf with comment stripping and line counting
 *
 *  (c) 1998 ESA  -  Written by Jon Brumfitt, Aurora
 *                -  Exported by Jorgo Bakker, BSC to layout package
 *
 *  $Id: CommentBuf.h,v 1.1 2000/04/14 09:39:50 jbakker Exp $
 *
 *****************************************************************/

#ifndef COMMENTBUF_H
#define COMMENTBUF_H

#include <iostream.h>          // C++ headers
#include <string>


/*****************************************************************
 *  Class CommentBuf
 *
 *  streambuf with comment stripping and line counting
 *
 *  A CommentBuf is a kind of streambuf which strips comments from the 
 *  input stream and keeps track of the current line number for error
 *  handling.
 *
 *  Comments are introduced with a '#' symbol and extend to the end
 *  of the line. For example:
 *
 *       # This is a comment line
 *       value = 42   # A trailing comment
 *
 *  The classes Cistream, Cifstream and Cistrstream are based on 
 *  CommentBuf, instead of the normal streambuf. For example:
 *
 *       Cistream is("myFile");
 *       if (! (is >> x))
 *           cerr << "Error at line " << is.lineNumber << endl
 *                << is.line() << endl;
 *
 *  A CommentBuf may also be created explicitly, as in the following 
 *  example:
 *
 *       ifstream iss("data");
 *       CommentBuf gpb(iss.rdbuf());
 *       istream is(&gpb);
 *
 *  In this case, it would be simpler to use Cistrstream:
 *
 *      Cistrstream("data");
 *
 *****************************************************************/

class CommentBuf : public streambuf
{
public:

    // Create a CommentBuf using a specified streambuf
    CommentBuf(streambuf *buf, bool delBuf = false);

    // Destructor
    ~CommentBuf();
  
    // Return the current line
    virtual string line() const;

    // Return number of the current line
    int lineNumber() const;

protected:

    // Fetch more data on buffer underflow
    int   underflow();

    // Enlarge the buffer and return pointer to previous end point.
    // Hence the buffer dynamically resizes to accomodate the longest line.
    char* enlargeBuffer();

private:

    // Member data
    streambuf*    _sbuf;
    bool          _autoDel;
    long          _lineLength;
    unsigned long _lineNumber;
};

#endif
