/*****************************************************************************
 *
 *  LYTReader.cc                                XMM Science Analysis Subsystem
 *
 *  Reader for ".lyt" layout files
 *
 *  (c) 1998  ESA  -  Written by Jon Brumfitt, Aurora
 *                 -  Exported by Jorgo Bakker, BSC to layout package
 *
 *  $Id: LYTReader.cc,v 1.2 2000/06/21 12:59:26 jbakker Exp $
 *
 *****************************************************************************/

#include <iostream.h>
#include <string>

#include <errstr.h>
#include <QuotedString.h>
#include <Cistream.h>

#include "LYTReader.h"
#include "Parameter.h"
#include "BoolParameter.h"
#include "Layout.h"


//================================================================
//  Class LYTReader
//================================================================

/*** Open layout file ***/

bool LYTReader::open(const string& filename)
{
   Cifstream* cis = new Cifstream(filename.c_str());
   _is = cis;
   _filename = filename;

   return *cis;
}


/*** Destructor ***/

LYTReader::~LYTReader()
{
    delete _is;
}


/*  Grammar for parser:
 *
 *  Layout       ::= PagedLayout | FrameLayout | EnableLayout 
 *                 | ChoiceLayout | RowLayout | ColumnLayout | Parameter
 *  LayoutList   ::= {Layout} "end"
 *  Parameter    ::= "Parameter" parname
 *  PagedLayout  ::= "Cards" {Page} "end"
 *  Page         ::= "Page" name {Layout} "end"
 *  FrameLayout  ::= "Frame" name {Layout} "end"
 *  EnableLayout ::= "Enable" parname {Layout} "end"
 *  ChoiceLayout ::= "Choice" parname {Page} "end"
 *  RowLayout    ::= "Row" {Layout} "end"
 *  ColumnLayout ::= "Column" {Layout} "end"
 */


// Layout ::= PagedLayout | FrameLayout | EnableLayout 
//          | ChoiceLayout | RowLayout | ColumnLayout | Parameter

bool LYTReader::parse(Layout*& lyt)
{
    bool status = false;

    string s;

    if(!((*_is) >> s))
    {
      /* Allow a layout file to be empty */
      errstr << "End of file reached without any layout specified" << endl
	     << "in layout file " << _filename << endl
	     << "at line " << _is->lineNumber()      
	     << Sas::message(Msg::Lib, Msg::Sparse);
      return false;
    }

    /* Parse the items */
    Layout* l;
    if(s == "Parameter")
        status = parseParameter(l);
    else if(s == "Cards")
        status = parsePagedLayout(l);
    else if (s == "Frame")
        status = parseFrameLayout(l);
    else if (s == "Enable")
        status = parseEnableLayout(l);
    else if (s == "Choice")
        status = parseChoiceLayout(l);
    else if (s == "Row")
        status = parseRowLayout(l);
    else if (s == "Column")
        status = parseColumnLayout(l);
    else if (s == "end")
        status = false;
    else {
        errstr << "Layout keyword not recognised: " << s << endl
	       << "in layout file " << _filename << endl
 	       << "at line " << _is->lineNumber()
	       << Sas::error("LytUnknownKeyword");
	_ok = false;
	return false;
    }

    lyt = l;
    return status;
}


// Parameter ::= "Parameter" parname

bool LYTReader::parseParameter(Layout*& lyt)
{
    QuotedString parname;
    (*_is) >> parname;

    Parameter* pm = _info->parameter(parname);
    if(!pm)
    {
        errstr << "Parameter '" << parname << "' is undefined" << endl
	       << "in layout file " << _filename << endl
 	       << "at line " << _is->lineNumber()
	       << Sas::error("LytUndefinedKeyword");
	_ok = false;
	return false;
    }
    SimpleLayout* sl = new SimpleLayout(pm);
    lyt = sl;
    return _ok;
}


// PagedLayout ::= "Cards" {Page} "end"

bool LYTReader::parsePagedLayout(Layout*& lyt)
{
    PagedLayout* layout = new PagedLayout;
    Layout* child;
    string name;
    while(parsePage(child,name))
    {
        child->setName(name);
        layout->addItem(child);
    }
    lyt = layout;
    return _ok;
}


// Page ::= "Page" name {Layout} "end"

bool LYTReader::parsePage(Layout*& lyt, string& name)
{
    ColumnLayout* layout = new ColumnLayout;
    Layout* child;

    string s;
    (*_is) >> s;
    if(s == "Page")
    {
        QuotedString pageName;
	(*_is) >> pageName;
	while(parse(child))
	    layout->addItem(child);

	lyt = layout;
	name = pageName;
	return _ok;
    }
    else if(s == "end")
        return false;
    else {
        errstr << "Page expected at '" << s << '\'' << endl
	       << "in layout file " << _filename << endl
 	       << "at line " << _is->lineNumber()
	       << Sas::error("LytPage");
	_ok = false;
        return false;
    }
}


// FrameLayout ::= "Frame" name {Layout} "end"

bool LYTReader::parseFrameLayout(Layout*& lyt)
{
    QuotedString name;
    (*_is) >> name;

    FrameLayout* layout = new FrameLayout;
    layout->setName(name);
    Layout* child;
    while(parse(child))
        layout->addItem(child);

    lyt = layout;

    return _ok;
}


// EnableLayout ::= "Enable" parname {Layout} "end"

bool LYTReader::parseEnableLayout(Layout*& lyt)
{
    QuotedString cntlname;
    (*_is) >> cntlname;

    Parameter* cntl = _info->parameter(cntlname);
    if(!cntl)
    {
        errstr << "Parameter '" << cntlname << "' is undefined" << endl
	       << "in layout file " << _filename << endl
 	       << "at line " << _is->lineNumber()
	       << Sas::error("LytUndefinedKeyword");
	_ok = false;
	return false;
    }

    if(cntl->getType() == Parameter::Bool)
    {
        EnableLayout* layout = new EnableLayout((BoolParameter*)cntl);
	Layout* child;
	while(parse(child))
	   layout->addItem(child);
	lyt = layout;
    }
    else
    {
        errstr << "Parameter '" << cntlname
	       << "' has wrong type for control" << endl
	       << "in layout file " << _filename << endl
 	       << "at line " << _is->lineNumber()
	       << Sas::error("cntlType");

	lyt = new ColumnLayout;  // Dummy, until we handle error properly
	_ok = false;
	return false;
    }

    return _ok;
}


// ChoiceLayout ::= "Choice" parname {Page} "end"

bool LYTReader::parseChoiceLayout(Layout*& lyt)
{    
    QuotedString cntlname;
    (*_is) >> cntlname;

    string name;
    Layout* child;

    Parameter* cntl = _info->parameter(cntlname);
    if(!cntl)
    {
        errstr << "Parameter '" << cntlname << "' is undefined" << endl
	       << "in layout file " << _filename << endl
 	       << "at line " << _is->lineNumber()
	       << Sas::error("LytUndefinedKeyword");
	_ok = false;
	return false;
    }
    if(cntl->getType() == Parameter::StringChoice)
    {
        ChoiceLayout* layout;
        layout = new ChoiceLayout((StringParameter*)cntl);
	while(parsePage(child,name))
	{
	    child->setName(name);
	    layout->addItem(child);
	}
	lyt = layout;
    }
    else
    {
	errstr << "Parameter '" << cntlname
	       << "' has wrong type for control" << endl
	       << "in layout file " << _filename << endl
 	       << "at line " << _is->lineNumber()
	       << Sas::error("LytCntlType");

	lyt = new ColumnLayout;  // Dummy, until we handle error properly
	_ok = false;
	return false;
    }
    return _ok;
}


// RowLayout ::= "Row" {Layout} "end"

bool LYTReader::parseRowLayout(Layout*& lyt)
{
    RowLayout* layout = new RowLayout;

    Layout* child;
    while(parse(child))
        layout->addItem(child);

    lyt = layout;

    return _ok;
}


// ColumnLayout ::= "Column" {Layout} "end"

bool LYTReader::parseColumnLayout(Layout*& lyt)
{
    ColumnLayout* layout = new ColumnLayout;

    Layout* child;
    while(parse(child))
        layout->addItem(child);

    lyt = layout;

    return _ok;
}

