/*****************************************************************************
 *
 *  Layout.h                                   XMM Science Analysis Subsystem
 *
 *  Classes for GUI layout
 *
 *  (c) 1998  ESA  -  Written by Jon Brumfitt, Aurora
 *                 -  Exported by Jorgo Bakker, BSC to layout package
 *
 *  $Id: Layout.h,v 1.1 2000/04/14 09:39:51 jbakker Exp $
 *
 *****************************************************************************/

#ifndef LAYOUT_H
#define LAYOUT_H

#include "Layout.h"
#include "LayoutBuilder.h"
#include "Parameter.h"
#include "BoolParameter.h"
#include "IntParameter.h"
#include "StringParameter.h"

#include <string>
#include <list.h>
#include <iostream.h>


//================================================================
//  Abstract class Layout
//================================================================

class Layout
{
public:
    virtual ~Layout() {}

    virtual void  build(LayoutBuilder* b) = 0;
    void setName(const string& name) { _name = name; }
    const string& name() const { return _name; }

private:
    string _name;
};


//================================================================
//  Class SimpleLayout
//
//  Wrapper for primitive parameters
//================================================================

class SimpleLayout : public Layout
{
public:
    SimpleLayout(Parameter* p) { _param = p; };
    void build(LayoutBuilder* b) { b->buildSimple(this); }
    Parameter* param() { return _param; }

private:
    Parameter* _param;
};


//================================================================
//  Class SequenceLayout
//
//  Base class for layouts which have a sequence of child layouts
//================================================================

class SequenceLayout : public Layout
{
public:
    virtual ~SequenceLayout();
    void addItem(Layout* l) { _items.push_back(l); }
    const list<Layout*>& items() const { return _items; }

private:
    list<Layout*> _items;
};

inline SequenceLayout::~SequenceLayout()
{
    list<Layout*>::iterator it = _items.begin();
    while(it != _items.end())
        delete *it++;
}


//================================================================
//  Class PagedLayout
//================================================================

class PagedLayout : public SequenceLayout
{
public:
    void build(LayoutBuilder* b) { b->buildCards(this); }
};


//================================================================
//  Class RowLayout
//================================================================

class RowLayout : public SequenceLayout
{
public:
    void build(LayoutBuilder* b) { b->buildRow(this); }
};


//================================================================
//  Class ColumnLayout
//================================================================

class ColumnLayout : public SequenceLayout
{
public:
    void build(LayoutBuilder* b) { b->buildColumn(this); }
};


//================================================================
//  Class FrameLayout
//================================================================

class FrameLayout : public SequenceLayout
{
public:
    void build(LayoutBuilder* b) { b->buildFrame(this); }
};


//================================================================
//  Class EnableLayout
//================================================================

class EnableLayout : public SequenceLayout
{
public:
    EnableLayout(BoolParameter* p) : _control(p) {}
    void build(LayoutBuilder* b) { b->buildEnable(this); }
    BoolParameter* control() const { return _control; }

private:
    BoolParameter* _control;
};


//================================================================
//  Class ChoiceLayout
//================================================================

class ChoiceLayout : public SequenceLayout
{
public:
    ChoiceLayout(StringParameter* p) : _control(p) {}
    void build(LayoutBuilder* b) { b->buildChoice(this); }
    StringParameter* control() const { return _control; }

private:
    StringParameter*  _control;
};


#endif
